""" 
	tinyLiDAR Terminal GUI for the ESP32 written in micropython

	Provides a simple user interface for most commands available in tinyLiDAR 
	
	Tested on wipy3.0 
	v0.95 to be use with tinyLiDAR f/w 1.4.1 or higher
	Last Edit: March 24, 2018 

	by Dinesh Bhatia MicroElectronicDesign, Inc. 
	https://www.microed.co 
	This work is licensed under Attribution-ShareAlike 3.0 Unported (CC BY-SA 3.0)
	https://creativecommons.org/licenses/by-sa/3.0/legalcode
	
Wiring details:

wipy3.0  to  tinyLiDAR
	P9 = SDA (White)
	P10 = SCL (Yellow)
	3V3 = V+ (Red)
	GND = GND (Black)

	RST pin momentary connection to GND as required for f/w update of wipy3.0 only

	Vin from +3.3v up to +5v power supply capable of supplying 300mA or more 
		(3.3v at 116mA nominal drain was measured in lab)
	GND = GND from power supply 

wipy3.0 program port:
	RX0(P0) From Tx pin of Serial USB cable
	TX0(P1) to Rx pin of Serial USB cable
"""

import sys, machine, gc, utime, struct 
from machine import Pin, I2C
import tinyLiDAR_var as tlv
import tinyLiDAR_menu as tlm
from tinyLiDAR_class import tinyLiDAR
gc.collect()

_timeBudget = tlv.TimingBudget_max  

RTmode = 1  # RTmode = -1 for RealTime mode else 1

i2c = I2C(0, I2C.MASTER, baudrate=100000) 
tof = tinyLiDAR(i2c)  # 0x10 is the default i2c address

tlm.printMenu()
tlm.printMenu1()

try:
	while True:
		print(u"\u001b[40A\u001b[90D\u001b[19B                                                   \u001b[1A") # position cursor
		commandEntered = input(u"> ") 
		i = commandEntered.upper() 

		tlm.home(2) # position cursor
		if  (i == '' or i == 'D'): 
			print(" %s" % tlv.tinyLiDAR_commands['D'])		
			tof.Dcommand(i2c, 1, _timeBudget * RTmode, tlv.I2C_Address )
			continue

		elif i == 'RESET':
			print(" %s" % tlv.tinyLiDAR_commands['RESET'])
			tlv.I2C_Address = tlv.default_tinyLiDAR_address #set to default I2C addr if no value entered 
			try:
				i2c.writeto(0, 6) # reset command    
			except:
				pass
			finally:
				machine.reset()

		elif i.startswith('I'):
			print(" %s" % tlv.tinyLiDAR_commands['I'])
			if (i == "I"):
				tlv.I2C_Address = tlv.default_tinyLiDAR_address #set to default I2C addr if no value entered 
				print (" --> New I2C Address for Terminal = 0x%X" % tlv.I2C_Address)
			else:
				tlv.I2C_Address = int(i[1:len(i)], 16)
				print (" --> New I2C Address for Terminal = 0x%X" % tlv.I2C_Address)
			tlm.printMenu()
			tlm.printMenu1()
			continue

		elif i.startswith('R'):
			print(" %s" % tlv.tinyLiDAR_commands['R'])
			if (i == "R"):
				i2cAdr = tlv.default_tinyLiDAR_address #set to default I2C addr if no value entered 
				print (" --> New I2C Address for tinyLiDAR = 0x%X" % i2cAdr)
			else:
				i2cAdr = int(i[1:len(i)], 16)
				print (" --> New I2C Address for tinyLiDAR = 0x%X" % i2cAdr)
			tof.I2C_Write(i2c, tlv.I2C_Address,0x52,i2cAdr) # is the R command
			continue 

		elif i == 'AZ':  
			print(" %s" % tlv.tinyLiDAR_commands['AZ'])
			i2c.deinit() 
			_SCLpin = Pin('P10', mode=Pin.OUT, pull=Pin.PULL_UP) # P10 is SCL for pycom	
			_SDApin = Pin('P9', mode=Pin.OUT, pull=Pin.PULL_UP) # P9 is SDA for pycom				]
			_SDApin(0)
			_SCLpin(0)
			utime.sleep_ms(1500)	
			i2c = I2C(0, I2C.MASTER, baudrate=100000) 
			tlm.home(2)
			input("> Press Enter to continue...")
			tlm._clrScrn()
 			continue

		elif i == 'CD':
			RTmode = 1 # Real Time mode is off
			tlm._clrScrn()
			j = _timeBudget  # temp storage
			_timeBudget = 215

			tlm.home(2)
			print(" %s" % tlv.tinyLiDAR_commands['CD'])

			tof.I2C_Write(i2c, tlv.I2C_Address, 'T0') #turn off the WDT
			utime.sleep_ms(tlv.rebootTime)			  #wait for reboot to continue			

			tof.I2C_Write(i2c, tlv.I2C_Address, 'PH')   #set to high accuracy preset
			utime.sleep_ms(tlv.rebootTime)		

			tlm.home(2)
			print(" : Distance Before Calibration ")

			tlm.home_clr(3) 
			for i in range(0, 10):
				tof.Dcommand(i2c, 1, 0, tlv.I2C_Address )
				utime.sleep_ms(_timeBudget+tlv._bufferTime) 

			caloffset = []
			caloffset.append(tlv.cal_offset_distance>>8 & 0x00ff) 
			caloffset.append(tlv.cal_offset_distance & 0x00ff) 

			tof.I2C_Write(i2c, tlv.I2C_Address, bytearray( [0x43] + [0x44] + caloffset) )  # is CD command

			tlm.home(2)
			print(" >>> Calibrating Now... please wait approx 10sec \n\r")
			utime.sleep(11)  # give it over 10sec to calibrate now

			tof.I2C_Write(i2c, tlv.I2C_Address, 'X')  #reboot tinyLiDAR 
			utime.sleep_ms(tlv.rebootTime)			  #wait for reboot to continue			

			tlm._clrScrn()
			tlm.home(2)
			print(" : Distance After Calibration ")

			tlm.home_clr(3) 
			for i in range(0, 10):
				tof.Dcommand(i2c, 1, 0, tlv.I2C_Address )
				utime.sleep_ms(_timeBudget+tlv._bufferTime) 

			tof.I2C_Write(i2c, tlv.I2C_Address, 'MS')  #set to single step mode again
			utime.sleep_ms(tlv.rebootTime)			  #wait for reboot to continue			

			tlm.home(2)
			input("> Press Enter to continue...")
			tlm._clrScrn()
			_timeBudget = j  # restore from temp storage
 			continue

		elif i == 'CX':
			RTmode = 1 # Real Time mode is off
			tlm._clrScrn()

			j = _timeBudget  # temp storage
			_timeBudget = 215

			tlm.home(2)
			print(" %s" % tlv.tinyLiDAR_commands['CX'])

			tof.I2C_Write(i2c, tlv.I2C_Address, 'T0') #turn off the WDT
			utime.sleep_ms(tlv.rebootTime)			  #wait for reboot to continue			

			tof.I2C_Write(i2c, tlv.I2C_Address, 'PH')   #set to high accuracy preset
			utime.sleep_ms(tlv.rebootTime)		

			tlm.home(2)
			print(" : Distance Before Calibration ")

			tlm.home_clr(3) 
			for i in range(0, 10):
				tof.Dcommand(i2c, 1, _timeBudget * RTmode, tlv.I2C_Address )

			caloffset = []
			caloffset.append(tlv.cal_xtalk_distance>>8 & 0x00ff) 
			caloffset.append(tlv.cal_xtalk_distance & 0x00ff) 

			tof.I2C_Write(i2c, tlv.I2C_Address, bytearray( [0x43] + [0x58] + caloffset) )  # is CX command

			tlm.home(2)
			print(" >>> Calibrating Now... please wait approx 10sec \n\r")
			utime.sleep(11)  # give it over 10sec to calibrate now

			tof.I2C_Write(i2c, tlv.I2C_Address, 'X')  #reboot tinyLiDAR 
			utime.sleep_ms(tlv.rebootTime)			  #wait for reboot to continue			

			tlm._clrScrn()
			tlm.home(2)
			print(" : Distance After Calibration ")

			tlm.home_clr(3) 
			for i in range(0, 10):
				tof.Dcommand(i2c, 1, _timeBudget * RTmode, tlv.I2C_Address )

			tof.I2C_Write(i2c, tlv.I2C_Address, 'MS')  #set to single step mode again
			utime.sleep_ms(tlv.rebootTime)			   #wait for reboot to continue			

			tlm.home(2)
			input("> Press Enter to continue...")
			tlm._clrScrn()
			_timeBudget = j  # restore from temp storage
 			continue

		elif i.startswith('B'):
			print(" %s" % tlv.tinyLiDAR_commands['B'])
			param = []
			if (i == "B"):    #set to default if no values are entered
				print("> Nothing entered - using Default Values: ") 
				if (tlv.Op_Mode_B == 1):
					print("   Object Detection Mode ")
					param.append(0xff) 
				else:
					print("   State Tracking Mode ")
					param.append(0) 
				
				print("   Low side limit = %d mm" % tlv.LowLimit_B ) 
				param.append( (tlv.LowLimit_B & 0xff00) >> 8 )   #MSB
				param.append (tlv.LowLimit_B) #LSB

				print("   High side limit = %d mm" % tlv.HighLimit_B ) 
				param.append( (tlv.HighLimit_B & 0xff00) >> 8)  #MSB
				param.append( tlv.HighLimit_B)  #LSB

				print("   Delay between measurements = %d ms" % tlv.Delay_Btwn_B ) 				
				param.append( (tlv.Delay_Btwn_B & 0xff00) >> 8)  #MSB
				param.append( tlv.Delay_Btwn_B)  #LSB

				if (tlv.LEDind_B == True):
					print("   LED indicator = ON   ")
					param.append(0xff) 
				else:
					print("   LED indicator = OFF   ")
					param.append(0) 
			else:
				print("> The values you entered were - ") 
				i = i.split(" ") 

				if (int(i[1]) == 1):
					print("   Object Detection Mode ")
					param.append(0xff) 
				else:
					print("   State Tracking Mode ")
					param.append(0) 
				 
				print("   Low side limit = %d mm" % int(i[2]) ) 
				param.append( (int(i[2]) & 0xff00) >> 8 )   #MSB
				param.append (int(i[2])) #LSB

				print("   High side limit = %d mm" % int(i[3]) ) 
				param.append( (int(i[3]) & 0xff00) >> 8)  #MSB
				param.append( int(i[3]))  #LSB

				print("   Delay between measurements = %d ms" %  int(i[4]) ) 				
				param.append( ( int(i[4]) & 0xff00) >> 8)  #MSB
				param.append(  int(i[4]))  #LSB

				if ( int(i[5]) == 1):
					print("   LED indicator = ON   ")
					param.append(0xff) 
				else:
					print("   LED indicator = OFF   ")
					param.append(0) 

			if( tof.confirm("> Okay to write values? [Y/N] ") ):

				tof.I2C_Write(i2c, tlv.I2C_Address, 'X')  # reboot first
				utime.sleep_ms(tlv.rebootTime)					

				tof.I2C_Write(i2c, tlv.I2C_Address, 'MS')  # set to SS mode
				utime.sleep_ms(tlv.rebootTime)		

				print(": Now writing parameters to tinyLiDAR. ")

				tof.I2C_Write(i2c, tlv.I2C_Address, bytearray([0x42] + param) )  
				utime.sleep_ms(tlv.rebootTime)					

				print(": Done. ")
			else:
				pass
			input("> Press Enter to continue...")
			tlm._clrScrn()
			continue

		elif i.startswith('A'):
			print(" %s" % tlv.tinyLiDAR_commands['A'])
			param = []

			if (i == "A"):    #set to default if no values are entered
				print("> Nothing entered - using Default Values: ") 
				print("   Low side limit = %d mm" % tlv.LowLimit ) 
				param.append( (tlv.LowLimit & 0xff00) >> 8 )   #MSB
				param.append (tlv.LowLimit) #LSB

				print("   High side limit = %d mm" % tlv.HighLimit ) 
				param.append( (tlv.HighLimit & 0xff00) >> 8)  #MSB
				param.append( tlv.HighLimit)  #LSB

				print("   Repetition interval = %d ms" % int(tlv.RepetitionInt*100) ) 				
				param.append( (tlv.RepetitionInt & 0xff00) >> 8)  #MSB
				param.append( tlv.RepetitionInt)  #LSB

				print("   Pulse Width = %d ms" % int(tlv.PulseWidth*100) )				
				param.append( ( tlv.PulseWidth & 0xff00) >> 8)   #MSB
				param.append( tlv.PulseWidth) #LSB

				if(tlv.LEDind == True):
					param.append( 0xff )
					print("   LED indicator = ON" )
				else:
					param.append( 0 )
					print("   LED indicator = OFF" )					

			else:
				print("> The values you entered were - ") 
				i = i.split(" ") 

				print("   Low side limit = %d mm" % int(i[1]) ) 
				param.append( (int(i[1]) & 0xff00) >> 8) #MSB
				param.append( int(i[1]))  #LSB

				print("   High side limit = %d mm" % int(i[2]) ) 
				param.append( (int(i[2]) & 0xff00) >> 8)  #MSB
				param.append( int(i[2]) ) #LSB

				print("   Repetition interval = %d ms" % int(i[3])*100 ) 
				param.append( (int(i[3]) & 0xff00) >> 8)  #MSB
				param.append( int(i[3] ) ) #LSB

				print("   Pulse Width = %d ms" % int(i[4])*100 ) 
				param.append( ( int(i[4]) & 0xff00) >> 8)   #MSB
				param.append(  int(i[4]) ) #LSB

				if (i[5] == 1):
					print("   LED indicator = ON" )
					param.append( 0xff )
				else: 
					print("   LED indicator = OFF" )
					param.append( 0)

			if( tof.confirm("> Okay to write values? [Y/N] ") ):
				tof.I2C_Write(i2c, tlv.I2C_Address, 'X')  # reboot first
				utime.sleep_ms(tlv.rebootTime)					

				tof.I2C_Write(i2c, tlv.I2C_Address, 'MS')  # set to SS mode
				utime.sleep_ms(tlv.rebootTime)		

				print(": Now writing parameters to tinyLiDAR. ")

				tof.I2C_Write(i2c, tlv.I2C_Address, bytearray([0x41] + param) )  
				utime.sleep_ms(tlv.rebootTime)					

				print(": Done. ")
			else:
				pass
			input("> Press Enter to continue...")
			tlm._clrScrn()
			continue

		elif i.startswith('W'):
			print(" %s" % tlv.tinyLiDAR_commands['W'])

			if (i == "W"):  #set to default if no values are entered
				param = []
				param.append( (tlv.SignalRateLimit & 0xff00) >> 8 )  #MSB
				param.append( tlv.SignalRateLimit)  #LSB
				param.append( tlv.SignalEstimateLimit) 
				param.append( (tlv.TimingBudget & 0xff00) >> 8) #MSB
				param.append( tlv.TimingBudget) #LSB
				param.append( tlv.VCELperiod) 
				print(":  Using Default Values - ") 

				i = (param[0]*256 + param[1]) / 100.0  
				print("   SignalLimit = %0.2f Mcps" % i ) 

				print("   SigmalLimit = %d mm" % param[2] ) 

				i = (param[3]*256 + param[4])
				print("   timeBudget = %d ms" % i ) 

				if (param[5] == 14): 
					print("   preRangeVcselPeriod = 14 ") 
					print("   finalRangeVcselPeriod = 10 ") 
				elif (param[5] == 18): 
					print("   preRangeVcselPeriod = 18 ") 
					print("   finalRangeVcselPeriod = 14 ") 

			else:
				param = []

				print("> The values you entered were - ") 

				i = i.split(" ") 
				print("   SignalLimit = %.2f Mcps" % (int(i[1])/100) ) 
				param.append((int(i[1]) & 0xff00) >> 8 )  #MSB
				param.append(int(i[1]) & 0x00ff)  #LSB

				print("   SigmalLimit = %d mm" % int(i[2]) ) 
				param.append(int(i[2]))

				print("   timeBudget = %d ms" % int(i[3]) ) 
				param.append(( int(i[3]) & 0xff00) >> 8 )  #MSB
				param.append( int(i[3]) & 0x00ff )  #LSB

				if (int(i[4]) == 14): 
					print("   preRangeVcselPeriod = 14 ") 
					print("   finalRangeVcselPeriod = 10 ") 
				elif (int(i[4]) == 18): 
					print("   preRangeVcselPeriod = 18 ") 
					print("   finalRangeVcselPeriod = 14 ") 
	                 
				param.append( int(i[4]) )

			if(tof.confirm()):
				print(": Now writing parameters to tinyLiDAR. ")

				tof.I2C_Write(i2c, tlv.I2C_Address, 'MS')  # set to SS mode
				utime.sleep_ms(tlv.rebootTime)					

				tof.I2C_Write(i2c, tlv.I2C_Address, bytearray([0x57] + param) )  
				utime.sleep_ms(tlv.rebootTime)					

				print(": Done.")

			else:
				pass
			input("> Press Enter to continue...")
			tlm._clrScrn()
			continue

		elif i == 'PL':
			_timeBudget = 33 
			print(" %s" % tlv.tinyLiDAR_commands[i])
			tof.I2C_Write(i2c, tlv.I2C_Address,i)		
			continue			

		elif i == 'PS':
			_timeBudget = 20 
			print(" %s" % tlv.tinyLiDAR_commands[i])
			tof.I2C_Write(i2c, tlv.I2C_Address,i)		
			continue			

		elif i == 'PH':
			_timeBudget = 200 
			print(" %s" % tlv.tinyLiDAR_commands[i])
			tof.I2C_Write(i2c, tlv.I2C_Address,i)		
			continue			

		elif i == 'PT':
		 	_timeBudget = 18
			print(" %s" % tlv.tinyLiDAR_commands[i])
			tof.I2C_Write(i2c, tlv.I2C_Address,i)		
			continue			

		elif i == 'DC':
		 	tof.ContinuousReadingForTerminal(i2c, tlv.I2C_Address, _timeBudget) 
		 	continue			

		elif i == 'MR':
		 	RTmode = -1  # -1 means real time mode is active
		 	tof.I2C_Write(i2c, tlv.I2C_Address,i)		
		 	print(" - Real Time mode active - ")
		 	continue			

		elif i == 'MS':
		 	RTmode = 1 # real time mode is off
		 	tof.I2C_Write(i2c, tlv.I2C_Address,i)				 	
		 	print(" - SingleStep/UltraLow Power mode active - ")
		 	continue			

		elif i == 'UZ':
			print(" %s" % tlv.tinyLiDAR_commands[i])
			print("  >> Press & Release RESET on tinyLiDAR now before count expires << ")

			i2c.deinit() 
			_SCLpin = Pin('P10', mode=Pin.OUT, pull=Pin.PULL_UP) 
			_SDApin = Pin('P9', mode=Pin.OUT, pull=Pin.PULL_UP)  
			_SCLpin(0) #send low 
			_SDApin(0) #send low 

			for j in range(11,0,-1):
				print(j-1)
				utime.sleep_ms(200)	#give it time to reboot and sense the lines being held low 
			tlm._clrScrn()
			i2c = I2C(0, I2C.MASTER, baudrate=100000) 
			continue

		elif i == 'Q':
			data = tof.Qcommand(i2c,tlv.I2C_Address);
			tlm.printMenu()
			if (RTmode == -1):
				print(">> Sorry, cannot run the Q command in this mode. ")
				utime.sleep(3)	
			 	tlm._clrScrn()

			else:
				sys.stdout.write(u"\u001b[1B  Current Operation Mode is: ") 
				if (data[0] == 0x43):  
					print(u"\u001b[1mContinuous\u001b[0m")
				elif (data[0] == 0x53):
					print(u"\u001b[1mSingleStep\u001b[0m")
				elif (data[0]== 0x4c):
					print(u"\u001b[1mUltraLowPower\u001b[0m")
				if ( (data[14] & 0x01) == 0 ):  
					print(u"  WatchDog Timer: \u001b[1mOFF\u001b[0m")
				else:
					print(u"  WatchDog Timer: \u001b[1mON\u001b[0m")
				i = (data[14] & 0x06) >> 1 #take the 2 bits for LED Indicator
				if (i == 0 ):  
					print(u"  LED Indicator: \u001b[1mOFF\u001b[0m")
				elif (i == 1 ): 
					print(u"  LED Indicator: \u001b[1mON\u001b[0m")
				elif (i == 2 ): 
					print(u"  LED Indicator: \u001b[1mMeasurement\u001b[0m")
				sys.stdout.write(u"  Current Preset Configuration is: ")
				if (data[1] == 0x53):
					print(u"\u001b[1mHigh Speed\u001b[0m")
				elif (data[1] == 0x52):
					print(u"\u001b[1mLong Range\u001b[0m")
				elif (data[1] == 0x41):
					print(u"\u001b[1mHigh Accuracy\u001b[0m")
				elif (data[1] == 0x43):
					print(u"\u001b[1mCustom\u001b[0m")
				elif (data[1] == 0x54):
					print(u"\u001b[1mtinyLiDAR\u001b[0m")
				else:
					print(u"\u001b[1m** UNKNOWN **\u001b[0m")
				sys.stdout.write(u"  Signal Rate Limit = ") 
				k = ((data[2]) << 8 | data[3])
				k = k / 65536.0 
				print(u"\u001b[1m%.2f MCPS\u001b[0m" % k)
				sys.stdout.write(u"  Sigma Estimate Limit = ") 
				print(u"\u001b[1m%d mm\u001b[0m" % data[4])
				sys.stdout.write(u"  Timing Budget = ") 
				k = ((data[5]) << 8 | data[6])
				print(u"\u001b[1m%d ms\u001b[0m" % k)
				if(data[7]) == 14:
					print(u"  Pre Range VCSEL Period = \u001b[1m14\u001b[0m ")  
					print(u"  Final Range VCSEL Period = \u001b[1m10\u001b[0m ")  
				else:
					print(u"  Pre Range VCSEL Period = \u001b[1m18\u001b[0m ")  
					print(u"  Final Range VCSEL Period = \u001b[1m14\u001b[0m ")  
				sys.stdout.write(u"  tinyLiDAR Firmware Version = ")  
				print(u"\u001b[1m"+str(data[8]) + "." + str(data[9]) + "." + str(data[10]) +u"\u001b[0m")
				sys.stdout.write(u"  ST PAL API Version = ")  
				print(u"\u001b[1m"+str(data[11]) + "." + str(data[12]) + "." + str(data[13])+u"\u001b[0m")
				if(data[14] & 0x08):
					print(u"  Offset Cal: \u001b[1mCustom\u001b[0m")  
				else:
					print(u"  Offset Cal: \u001b[1mFactory\u001b[0m")  
				
				sys.stdout.write(u"  Cal Offset = ")  
				ilong = (data[15]<<24 |data[16]<<16 | data[17]<<8 | data[18])
				x = struct.unpack('i', struct.pack('I',ilong ) )[0]  #convert hex to signed int
				print(u"\u001b[1m%d mm\u001b[0m" % (x/1000.0) ) 

				sys.stdout.write(u"  Xtalk = ")  
				ilong = (data[19]<<24 |data[20]<<16 | data[21]<<8 | data[22])
				x = struct.unpack('i', struct.pack('I',ilong ) )[0]  #convert hex to signed int
				print(u"\u001b[1m%.3f MCPS\u001b[0m" % (x/65536.0) )  
				
				input(u"\u001b[1B> Press ENTER to Continue ")
				tlm.printMenu()
				tlm.printMenu1()
			continue

		elif i in tlv.tinyLiDAR_commands:
			tlm.home_clr(2)
			print(" %s" % tlv.tinyLiDAR_commands[i])
			tof.I2C_Write(i2c, tlv.I2C_Address,i)		
			continue			
		else:
			print('  ** Command Not Valid, please try again **')

except KeyboardInterrupt:
	tlm._clrScrn()
	machine.reset()

# end of code	